/*
    \brief Deklarationen der Listenfunktionen zur Zutrittskontrolle Version 2

    \author Bernd Ottmann
    \date   21.02.2003
*/
#pragma once
#ifndef DATAFOX_ACCESS_CONTROL_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define DATAFOX_ACCESS_CONTROL_H
#endif

#include "df_defines.h"

#if (DF_API_VERSION >= 0x04010400)

/*!
    \brief Löst eine virtuelle Zutrittsbuchung in der Zutrittskontrolle des Gerätes aus.

    Die Funktion lässt sich aus Sicherheitsgründen nur auf Ausweise anwenden die im Feld "ActiveGeneral"
    der Liste "Identification" entweder den Wert "2" oder "9" eingetragen haben.

    \ingroup access_control_v2

    \export{_DFCEntrance2Identification@32, DFCEntrance2Identification}

    \deprecated Bitte verwenden Sie wenn möglich DFCAccessControlIdentification(). Diese biedet Ihnen zusätzlich die
    Angabe eines speziellen Zutrittsbusses, wenn mehrere Zutrittsstränge verfügbar sein sollten.

    \note Nähere Informationen für die zu übergebenden Feldwerte (TM, Id, Pin) sowie den zurückgelieferten Status (\a statusText)
    entnehmen Sie bitte dem jeweilingen Handbuch aus dem Kapitel "Zutrittskontrolle" und Unterkapitel "Statusmeldungen".

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] readerTM Feldwert entsprechend des Feldes "TM" der Liste "Reader".
    Er muss demnach immer 3 Ziffern umfassen.
    \param[in] identificationId Feldwert entsprechend des Feldes "Id" der Liste "Identification".
    Er darf maximal 20 Zeichen lang sein.
    \param[in] identificationPin Feldwert entsprechend des Feldes "Pin" der Liste "Identification".
    Er darf maximal 8 Ziffern beinhalten.
    \param[in] statusText Zeichenarray für den Status der Zutrittsprüfung.
    Sie sollten mindestens 5 Zeichen vorsehen.
    \param[in,out] statusTextLength Als \c in die Länge des Arrays von \a statusText, als \c out die Länge der übergebenen Zeichenfolge.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.04 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCEntrance2Identification( int connectionNumber, int deviceAddress, const char *readerTM, const char *identificationId, const char *identificationPin, char *statusText, int *statusTextLength, int *errorNumber );

#endif

#if (DF_API_VERSION >= 0x04010500)

/*!
    \brief Löst im Online-Modus der Zutrittskontrolle des Gerätes, eine Leser-Aktion durch.

    Steuern des Buzzer, LED und Relais eines Zutrittslesers im Online-Modus der Zutrittskontrolle.

    \ingroup access_control_v2

    \export{_DFCEntrance2OnlineAction@28, DFCEntrance2OnlineAction}

    \deprecated Bitte verwenden Sie wenn möglich DFCAccessControlOnlineAction(). Diese biedet Ihnen zusätzlich die
     Angabe eines speziellen Zutrittsbusses, wenn mehrere Zutrittsstränge verfügbar sein sollten.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] readerTM Feldwert entsprechend des Feldes "TM" der Liste "Reader".
    Er muss demnach immer 3 Ziffern umfassen.
    \param[in] mask Wert der anzusprechenden Komponenten. Als Ausgang kann je nach verwendetem Modul, ein Relais oder ein digitaler Ausgang vorliegen.
    <table boarder="0" width="100%">
    <tr><td>
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Buzzer</td></tr>
        <tr><td>2</td><td>2</td><td>Grüne LED</td></tr>
        <tr><td>3</td><td>4</td><td>Rote LED</td></tr>
        <tr><td>4</td><td>8</td><td>Ausgang 1</td></tr>
        <tr><td>5</td><td>16</td><td>Ausgang 2</td></tr>
        <tr><td>6</td><td>32</td><td>Ausgang 3</td></tr>
        <tr><td>7</td><td>64</td><td>Ausgang 4</td></tr>
        <tr><td>8</td><td>128</td><td>Gelbe LED</td></tr>
        <tr><td>9</td><td>256</td><td>Ausgang 5</td></tr>
        <tr><td>10</td><td>512</td><td>Ausgang 6</td></tr>
        <tr><td>11</td><td>1024</td><td>Ausgang 7</td></tr>
        <tr><td>12</td><td>2048</td><td>Ausgang 8</td></tr>
        <tr><td>13</td><td>4096</td><td>Ausgang 9</td></tr>
        <tr><td>14</td><td>8192</td><td>Ausgang 10</td></tr>
        <tr><td>15</td><td>16384</td><td>Ausgang 11</td></tr>
        <tr><td>16</td><td>32768</td><td>Ausgang 12</td></tr>
    </table>
    </td>
    <td>
        <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>17</td><td>65536</td><td>Ausgang 13</td></tr>
        <tr><td>18</td><td>131072</td><td>Ausgang 14</td></tr>
        <tr><td>19</td><td>262144</td><td>Ausgang 15</td></tr>
        <tr><td>20</td><td>524288</td><td>Ausgang 16</td></tr>
        <tr><td>21</td><td>1048576</td><td>Ausgang 17</td></tr>
        <tr><td>22</td><td>2097152</td><td>Ausgang 18</td></tr>
        <tr><td>23</td><td>4194304</td><td>Ausgang 19</td></tr>
        <tr><td>24</td><td>8388608</td><td>Ausgang 20</td></tr>
        <tr><td>25</td><td>16777216</td><td>Ausgang 21</td></tr>
        <tr><td>26</td><td>33554432</td><td>Ausgang 22</td></tr>
        <tr><td>27</td><td>67108864</td><td>Ausgang 23</td></tr>
        <tr><td>28</td><td>134217728</td><td>Ausgang 24</td></tr>
        <tr><td>29</td><td>268435456</td><td>Ausgang 25</td></tr>
        <tr><td>30</td><td>536870912</td><td>Ausgang 26</td></tr>
        <tr><td>31</td><td>1073741824</td><td>Ausgang 27</td></tr>
        <tr><td>32</td><td>2147483648</td><td>Ausgang 28</td></tr>
    </table>
    </td></tr>
    </table>
     Sie geben durch die gesetzten Bits an, welche Komponenten bei Ausführung angesprochen werden sollen.\n
     Als Beispiel geben Sie für das ansprechen der Grünen LED und des 9 Relais (2 + 4096 =) 4098 als Wert an.
    \param[in] type Anzunehmenden Zustand der gewählten Komponenten.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>Aus</td></tr>
        <tr><td>1</td><td>Ein</td></tr>
        <tr><td>2</td><td>Dauerhaft im Wechsel 600ms an, 600ms aus.</td></tr>
        <tr><td>3</td><td>Drei mal im Wechsel 400ms an, 400ms aus</td></tr>
    </table>
    \param[in] duration Zusätzliche Zeitangabe bei Angabe von \a type = 1. Ansonsten wird der Wert ignoriert.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>Dauerhaft</td></tr>
        <tr><td>1 bis 40</td><td>Anzahl Sekunden für den Zustand: Ein</td></tr>
    </table>
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.01.05 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCEntrance2OnlineAction( int connectionNumber, int deviceAddress, const char *readerTM, int mask, int type, int duration, int *errorNumber);

#endif

#if (DF_API_VERSION >= 0x04030403)

/*!
    \brief Löst eine virtuelle Zutrittsbuchung in der Zutrittskontrolle des Gerätes aus.

    Die Funktion lässt sich aus Sicherheitsgründen nur auf Ausweise anwenden die im Feld "ActiveGeneral"
    der Liste "Identification" entweder den Wert "2" oder "9" eingetragen haben.

    \ingroup access_control_v2

    \export{_DFCAccessControlIdentification@36, DFCAccessControlIdentification}

    \note Nähere Informationen für die zu übergebenden Feldwerte (TM, Id, Pin) sowie den zurückgelieferten Status (statusText)
    entnehmen Sie bitte dem jeweilingen Handbuch aus dem Kapitel "Zutrittskontrolle" und Unterkapitel "Statusmeldungen".

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] masterId Anzusprechende Master-Id. Diese entspricht dem Zutrittsbus bei Geräten mit mehreren Zutrittsstängen.
    Standardwert ist 1. Wertebereich 1 - 9999.
    \param[in] readerTM Feldwert entsprechend des Feldes "TM" der Liste "Reader".
    Er muss demnach immer 3 Ziffern umfassen.
    \param[in] identificationId Feldwert entsprechend des Feldes "Id" der Liste "Identification".
    Er darf maximal 20 Zeichen lang sein.
    \param[in] identificationPin Feldwert entsprechend des Feldes "Pin" der Liste "Identification".
    Er darf maximal 8 Ziffern beinhalten.
    \param[in] statusText Zeichenarray für den Status der Zutrittsprüfung.
    Sie sollten mindestens 5 Zeichen vorsehen.
    \param[in,out] statusTextLength Als \c in die Länge des Arrays für \a statusText, als \c out die Länge der übergebenen Zeichenfolge.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.03.04 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCAccessControlIdentification( int connectionNumber, int deviceAddress, int masterId, const char *readerTM, const char *identificationId, const char *identificationPin, char *statusText, int *statusTextLength, int *errorNumber );

/*!
    \brief Löst im Online-Modus der Zutrittskontrolle des Gerätes, eine Leser-Aktion aus.

    Steuern des Buzzer, LED und Relais eines Zutrittslesers im Online-Modus der Zutrittskontrolle.

    \ingroup access_control_v2

    \export{_DFCAccessControlOnlineAction@32, DFCAccessControlOnlineAction}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] masterId Anzusprechende Master-Id. Diese entspricht dem Zutrittsbus bei Geräten mit mehreren Zutrittsstängen.
    Standardwert ist 1. Wertebereich 1 - 9999.
    \param[in] readerTM Feldwert entsprechend des Feldes "TM" der Liste "Reader".
    Er muss demnach immer 3 Ziffern umfassen.
    \param[in] mask Wert der anzusprechenden Komponenten. Als Ausgang kann je nach verwendetem Modul, ein Relais oder ein digitaler Ausgang vorliegen.
    <table boarder="0" width="100%">
    <tr><td>
    <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>1</td><td>Buzzer</td></tr>
        <tr><td>2</td><td>2</td><td>Grüne LED</td></tr>
        <tr><td>3</td><td>4</td><td>Rote LED</td></tr>
        <tr><td>4</td><td>8</td><td>Ausgang 1</td></tr>
        <tr><td>5</td><td>16</td><td>Ausgang 2</td></tr>
        <tr><td>6</td><td>32</td><td>Ausgang 3</td></tr>
        <tr><td>7</td><td>64</td><td>Ausgang 4</td></tr>
        <tr><td>8</td><td>128</td><td>Gelbe LED</td></tr>
        <tr><td>9</td><td>256</td><td>Ausgang 5</td></tr>
        <tr><td>10</td><td>512</td><td>Ausgang 6</td></tr>
        <tr><td>11</td><td>1024</td><td>Ausgang 7</td></tr>
        <tr><td>12</td><td>2048</td><td>Ausgang 8</td></tr>
        <tr><td>13</td><td>4096</td><td>Ausgang 9</td></tr>
        <tr><td>14</td><td>8192</td><td>Ausgang 10</td></tr>
        <tr><td>15</td><td>16384</td><td>Ausgang 11</td></tr>
        <tr><td>16</td><td>32768</td><td>Ausgang 12</td></tr>
    </table>
    </td>
    <td>
        <table class="paramvalues">
        <tr><th>Bitnummer</th><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>17</td><td>65536</td><td>Ausgang 13</td></tr>
        <tr><td>18</td><td>131072</td><td>Ausgang 14</td></tr>
        <tr><td>19</td><td>262144</td><td>Ausgang 15</td></tr>
        <tr><td>20</td><td>524288</td><td>Ausgang 16</td></tr>
        <tr><td>21</td><td>1048576</td><td>Ausgang 17</td></tr>
        <tr><td>22</td><td>2097152</td><td>Ausgang 18</td></tr>
        <tr><td>23</td><td>4194304</td><td>Ausgang 19</td></tr>
        <tr><td>24</td><td>8388608</td><td>Ausgang 20</td></tr>
        <tr><td>25</td><td>16777216</td><td>Ausgang 21</td></tr>
        <tr><td>26</td><td>33554432</td><td>Ausgang 22</td></tr>
        <tr><td>27</td><td>67108864</td><td>Ausgang 23</td></tr>
        <tr><td>28</td><td>134217728</td><td>Ausgang 24</td></tr>
        <tr><td>29</td><td>268435456</td><td>Ausgang 25</td></tr>
        <tr><td>30</td><td>536870912</td><td>Ausgang 26</td></tr>
        <tr><td>31</td><td>1073741824</td><td>Ausgang 27</td></tr>
        <tr><td>32</td><td>2147483648</td><td>Ausgang 28</td></tr>
    </table>
    </td></tr>
    </table>
     Sie geben durch die gesetzten Bits an, welche Komponenten bei Ausführung angesprochen werden sollen.\n
     Als Beispiel geben Sie für das ansprechen der Grünen LED und des 9 Relais (2 + 4096 =) 4098 als Wert an.
    \param[in] type Anzunehmenden Zustand der gewählten Komponenten.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>Aus</td></tr>
        <tr><td>1</td><td>Ein</td></tr>
        <tr><td>2</td><td>Dauerhaft im Wechsel 600ms an, 600ms aus.</td></tr>
        <tr><td>3</td><td>Drei mal im Wechsel 400ms an, 400ms aus</td></tr>
    </table>
    \param[in] duration Zusätzliche Zeitangabe bei Angabe von \a type = 1. Ansonsten wird der Wert ignoriert.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>0</td><td>Dauerhaft</td></tr>
        <tr><td>1 bis 40</td><td>Anzahl Sekunden für den Zustand: Ein</td></tr>
    </table>
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.

    \since 04.03.04 enthalten
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCAccessControlOnlineAction( int connectionNumber, int deviceAddress, int masterId, const char *readerTM, int mask, int type, int duration, int *errorNumber );

#endif

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#if (DF_API_VERSION >= 0x04030500)


// \export{_DFCAccessControlKnobCommand@36, DFCAccessControlKnobCommand}
/*
    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[in] knobCommand Anzunehmenden Zustand der gewählten Komponenten.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>Teach</td></tr>
        <tr><td>2</td><td>List</td></tr>
        <tr><td>3</td><td>Battery change begin</td></tr>
        <tr><td>4</td><td>Battery change end</td></tr>
    </table>
    \param[in] masterId Anzusprechende Master-Id. Diese entspricht dem Zutrittsbus bei Geräten mit mehreren Zutrittsstängen.
    \param[in] knobNumber Anzusprechender Türknauf.
    \param[in] parms Nullterminierte Zusatzzeichenfolge. Derzeit nicht verwendet.
    \param[out] buffer Puffer für eine zurückzugebende Zeichenfolge. Bei \a knobCommand "List" wird hier die Information zu den Knäufen zurückgeliefert.
    \param[in, out] bufferSize Als Eingabe die Länge des Puffers von \a buffer, als Ausgabe die dahin kopierte Anzahl Zeichen.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.
*/ 
DF_API_EXTERN_DECORATE int DF_API_CALL DFCAccessControlKnobCommand( int connectionNumber, int deviceAddress, int knobCommand, int masterId, int knobNumber, const char *params, char *buffer, int *bufferSize, int *errorNumber );

#endif
#endif

#endif
