#include "api/df_access_control_lists_v1.h"
#include "datafox_library_iv.h"

DF_API_EXTERN_DECORATE void DF_API_CALL DFCClearEntranceListBuffer( int connectionNumber, int number )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    channelBridge->logBegin( __func__, "\t%i", number );

    // zulaessige Bereiche pruefen
    if ( number < -1 || number > 3 )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "" );
        return;
    }

    channelBridge->ClearEntranceListBuffer( number );

    channelBridge->logEnd( "" );
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCMakeEntranceList( int connectionNumber, int number, int recordCount, int size, const unsigned char *buffer, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, "\t%i\t%i\t%i", number, recordCount, size );

    // zulaessige Bereiche pruefen
    if ( number < 0 || number > 3 || buffer == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );

        channelBridge->logEnd( "\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS, errorNumber );

    // ListNum ist:
    // 0: Zeitzonen     (22 Byte) zu komprimierten  (6 Byte) max.   16 DS moeglich
    // 1: Zeitmodelle   (35 Byte) zu komprimierten  (3 Byte) max.   32 DS moeglich
    // 2: Personalstamm (19 Byte) zu komprimierten (16 Byte) max. 5120 DS moeglich
    // 3: Feiertage     (16 Byte) zu komprimierten  (6 Byte) max.   64 DS moeglich

    // Bereich der gueltigen Nummer pruefen.
    switch ( number )
    {
    case 0:
        if ( (recordCount < 1) || (recordCount > 16) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
            break;
        }

        if ( size != (recordCount * ENTRANCE_RECORD_SIZE_0) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
            break;
        }
        break;

    case 1:
        if ( (recordCount < 1) || (recordCount > 32) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
            break;
        }

        if ( size != (recordCount * ENTRANCE_RECORD_SIZE_1) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
            break;
        }
        break;

    case 2:
        if ( (recordCount < 1) || (recordCount > 5120) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
            break;
        }

        if ( size != (recordCount * ENTRANCE_RECORD_SIZE_2) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
            break;
        }
        break;

    case 3:
        if ( (recordCount < 1) || (recordCount > 64) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
            break;
        }

        if ( size != (recordCount * ENTRANCE_RECORD_SIZE_3) )
        {
            channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
            break;
        }
        break;

    default:
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        break;
    }


    int iRes = 0;
    if ( channelBridge->getLastErrorNumber( 0 ) == IV_ERROR_SUCCESS )
    {
        iRes = channelBridge->MakeEntranceList( number, recordCount, size, buffer );
    }

    channelBridge->logEnd("\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber(0));

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadEntranceList( int connectionNumber, int deviceAddress, int number, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "\t%i", number );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 || number < -1 || number > 3 )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_PARAMETER, errorNumber );

        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    int iRes;
    // Kommunikationsversionsnummer
    int commVersion = 0;

    if ( channelBridge->getProtocolVersion( l_device_address ) == 1 )
    {
        int l_error_number = IV_ERROR_SUCCESS;
        // Bei AEIII+ zusaetzliche Pruefung wegen Befehlsunterstuetzung notwendig
        commVersion = channelBridge->getProtocolVersion2( l_device_address, TRUE, &l_error_number );
        switch ( commVersion )
        {
            //case 0x0400: Unterstuetzt kein Zutritt
        case 0x0401:
        case 0x0402:
            break;

        default:
            channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_VERSION_FIRMWARE, errorNumber );

            channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
            return 0;
        }
    }

    if ( commVersion != 0x0400 && commVersion != 0x0401 )
    {
        std::string version;
        // Firmwareversion lesen
        if ( (iRes = channelBridge->getVersionFirmware( l_device_address, version )) != IV_ERROR_SUCCESS )
        {
            channelBridge->setLastErrorNumber( l_device_address, (iRes < 0) ? -iRes : IV_ERROR_VERSION_FIRMWARE, errorNumber );

            channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
            return 0;
        }

        // Firmwareversion pruefen
        if ( !theApp.m_util.CheckVersionFirmwareZKOld( version.c_str() ) )
        {
            channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );

            channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
            return 0;
        }
    }

    int l_error_number = IV_ERROR_SUCCESS;
    iRes = channelBridge->writeEntranceListData( l_device_address, number, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( l_device_address, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( l_device_address ) );

    return iRes;
}
