#include "api/df_connection.h"
#include "datafox_library_iv.h"

DF_API_EXTERN_DECORATE HANDLE DF_API_CALL DFCGetComPort( int connectionNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return INVALID_HANDLE_VALUE;
    }

    channelBridge->logBegin( __func__, "" );

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    HANDLE hRes = channelBridge->getComPort();

    channelBridge->logEnd( "\t(%i, %i)", hRes, channelBridge->getLastErrorNumber( 0 ) );

    return hRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetComPort( int connectionNumber, HANDLE handle, int isSocket, int value, int timeout )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "" );

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = (channelBridge->setComPort( handle, isSocket, value, timeout ) == IV_ERROR_SUCCESS);

    channelBridge->logEnd( "\t%i\t%i\t%i\t%i\t(%i, %i)", handle, isSocket, value, timeout, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComOpenSerial( int connectionNumber, const char *commPort, int baudRate, int timeout )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "\t%s\t%i\t%i", (commPort == nullptr) ? "" : commPort, baudRate, timeout );

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = 0;

    // Schnittstelle geschlossen?
    if ( channelBridge->isOpen() == false )
    {
        if ( channelBridge->openAEIII( 1, commPort, baudRate, timeout ) == IV_ERROR_SUCCESS )
        {
            iRes = TRUE;
        }
    }
    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComOpenSocket( int connectionNumber, const char *hostAddress, int portNumber, int timeout )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "\t%s\t%i\t%i", (hostAddress == nullptr) ? "" : hostAddress, portNumber, timeout );

    // zulaessige Bereiche pruefen
    if ( hostAddress == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = 0;

    // Schnittstelle geschlossen?
    if ( channelBridge->isOpen() == false )
    {
        if ( channelBridge->openAEIII( 3, hostAddress, portNumber, timeout ) == IV_ERROR_SUCCESS )
        {
            iRes = TRUE;
        }
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComOpenIV( int connectionNumber, int deviceAddress, int connectionType, const char *connectionString, int connectionValue, int comunicationReadTimeout )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "\t%i\t%i\t%s\t%i\t%i", deviceAddress, connectionType,
                             (connectionString == nullptr) ? "" : connectionString, connectionValue, comunicationReadTimeout );

    if ( channelBridge->tryEnterCriticalSection() == false )
    {
        channelBridge->logAddError( "Object in use. Owner-Id: %u", channelBridge->m_thread_id_open );
        channelBridge->enterCriticalSection();
    }

    // zulaessige Bereiche pruefen
    if ( (connectionType != 1 && connectionType != 3 && connectionType != 4) 
        || connectionString == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( 0 ) );
        channelBridge->leaveCriticalSection();
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = 0;

    // Schnittstelle geoschlossen?
    if ( channelBridge->isOpen() == false )
    {
        //2009_09_02: Busnummer fix auf 32
        //2012_05_21: Busnummer fix auf 0 (wegen Mehrfachdocking TimeboyIV)
        if ( channelBridge->open( 0, connectionType, connectionString, connectionValue, comunicationReadTimeout ) == IV_ERROR_SUCCESS )
        {
            iRes = TRUE;
            // Die Thread-Id halten und beim Schließen prüfen,
            // dass auch der gleiche Thread wieder die Verbindung beendet.
            channelBridge->m_thread_id_open = std::this_thread::get_id();
        }
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );
    channelBridge->leaveCriticalSection();

    return iRes;
}

DF_API_EXTERN_DECORATE void DF_API_CALL DFCComClose( int connectionNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return;
    }

    channelBridge->logBegin( __func__, "" );

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    if ( channelBridge->m_thread_id_open != std::thread::id() && channelBridge->m_thread_id_open != std::this_thread::get_id() )
    {
        if ( channelBridge->m_checkOpenAndCloseBySameThread )
        {
            channelBridge->logAddError( "Not object owner. Owner-Id: %u", __func__, channelBridge->m_thread_id_open );
        }
        else
        {
            channelBridge->logAddWarning( "Not object owner. Owner-Id: %u", __func__, channelBridge->m_thread_id_open );

            channelBridge->close();
            channelBridge->m_thread_id_open = std::thread::id();
        }
    }
    else
    {
        channelBridge->close();
        channelBridge->m_thread_id_open = std::thread::id();
    }

    channelBridge->logEnd( "" );
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCWrite( int connectionNumber, const char *buffer, int toWrite, int *written, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, "" );

    // zulaessige Bereiche pruefen
    if ( buffer == nullptr || written == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t\t%i\t\t%i\t(%i, %i)", toWrite, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_CHANNEL_CLOSED, errorNumber );     
        channelBridge->logEnd( "\t%i\t\t%i\t(%i, %i)", toWrite, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS, errorNumber );

    int iRes = 0;

    if ( channelBridge->write( (const uint8_t *)buffer, toWrite, (uint32_t &)*written ) == IV_ERROR_SUCCESS )
    {
        iRes = 1;
    }

    channelBridge->logEnd( "\t%s\t%i\t%i\t%i\t(%i, %i)", (strlen(buffer) <= static_cast<uint32_t>( toWrite )) ? buffer : "",
        toWrite, *written, (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCRead( int connectionNumber, char *buffer, int toRead, int *read, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, "" );

    // zulaessige Bereiche pruefen
    if ( buffer == nullptr || read == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t\t%i\t\t%i\t(%i, %i)", toRead, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t\t%i\t(%i, %i)", toRead, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS, errorNumber );

    int iRes = 0;

    if ( channelBridge->read( (uint8_t *)buffer, toRead, (uint32_t &)*read) == IV_ERROR_SUCCESS )
    {
        iRes = 1;
    }

    channelBridge->logEnd( "\t%s\t%i\t%i\t%i\t(%i, %i)", (strlen(buffer) <= static_cast<uint32_t>( toRead )) ? buffer : "",
        toRead, *read, (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCIsChannelOpen( int connectionNumber )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *channelBridge = theApp.channelBridge( id );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr ) 
    {
        return 0;
    }

    channelBridge->logBeginPure( __func__, "\t%i", connectionNumber );

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = -1;

    if ( connectionNumber > 0 && connectionNumber < MAX_CHANNEL )
    {
        std::lock_guard<std::recursive_mutex> guard( theApp.m_mutexMapChannelBridge );
        if ( theApp.m_mapChannelBridge.count( connectionNumber ) )
        {
            if ( theApp.m_mapChannelBridge[connectionNumber]->isOpen() )
            {
                iRes = 1;
            }
            else 
            {
                iRes = 0;
            }
        }
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComUseHandle( int connectionNumber, HANDLE commHandle )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "\t%i", commHandle );

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = (channelBridge->comUseHandle( commHandle ) == IV_ERROR_SUCCESS);

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComReleaseHandle( int connectionNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "" );

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = (channelBridge->comReleaseHandle() == IV_ERROR_SUCCESS);

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComInit( int connectionNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "" );

    char szComStr[20];
#if defined(_MSC_VER) || defined(__MINGW32__)
    if ( connectionNumber > 9 )
    {
        snprintf( szComStr, sizeof( szComStr ), "\\\\.\\COM%i", static_cast<uint32_t>(connectionNumber) );
    }
    else
    {
        snprintf( szComStr, sizeof( szComStr ), "COM%i", static_cast<uint32_t>(connectionNumber) );
    }
#else
    snprintf( szComStr, sizeof( szComStr ), "/dev/ttyS%i", static_cast<uint32_t>(connectionNumber - 1) );
#endif

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = 0;

    // Schnittstelle geschlossen?
    if ( channelBridge->isOpen() == false )
    {
        if ( channelBridge->openAEIII( 1, szComStr, 38400, 800 ) == IV_ERROR_SUCCESS )
        {
            iRes = TRUE;
        }
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCComOpen( int connectionNumber, const char *hostAddress )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBegin( __func__, "\t%s", (hostAddress == nullptr) ? "" : hostAddress );

    if ( hostAddress == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );

        channelBridge->logEnd( "\t(%i, %i)", 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int iRes = 0;

    // Schnittstelle geschlossen?
    if ( channelBridge->isOpen() == false )
    {
        if ( channelBridge->openAEIII( 3, hostAddress, 8000, 5000 ) == IV_ERROR_SUCCESS )
        {
            iRes = 1;
        }
    }

    channelBridge->logEnd( "\t(%i, %i)", iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

