#include "api/df_files.h"
#include "datafox_library_iv.h"
#include <vector>
#include <boost/algorithm/string/split.hpp>
#include "common/utils.h"

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSetupLaden( int connectionNumber, int deviceAddress, const char *filePath, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "\t%s", (filePath == nullptr) ? "" : filePath );

    // Geräteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 || filePath == nullptr || strlen( filePath ) < 5 )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );
    // Fehlerobjekt initalisieren.
    int32_t l_error_number = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Anderer Blocktransfer besteht bereits?
    if ( channelBridge->getBlockTransferState( l_device_address ) == ControllerSegment::BlockTransferState::active
        && channelBridge->getBlockTransferType( l_device_address ) != ControllerSegment::btt_write_setup_file )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_BLOCK_TRANSFER_ACTIVE, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Setupdaten uebertragen
    int iRes = channelBridge->writeStudioFile( l_device_address, filePath, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( l_device_address, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( l_device_address ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCDownload( int connectionNumber, int deviceAddress, const char *filePath, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "\t%s", (filePath == nullptr) ? "" : filePath );

    // Geräteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 || filePath == nullptr || strlen( filePath ) < 5 )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_CHANNEL_CLOSED, errorNumber );       
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Anderer Blocktransfer besteht bereits?
    if ( channelBridge->getBlockTransferState( l_device_address ) == ControllerSegment::BlockTransferState::active
        && channelBridge->getBlockTransferType( l_device_address ) != ControllerSegment::btt_read_setup_file )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_BLOCK_TRANSFER_ACTIVE, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    // Setupdaten aus Geraet holen
    int iRes = channelBridge->readStudioFile( l_device_address, filePath, &l_error_number );

    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( l_device_address, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 1, channelBridge->getLastErrorNumber( l_device_address ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCUpload( int connectionNumber, int deviceAddress, const char *filePath, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Geräteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 || filePath == nullptr )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", (filePath == nullptr) ? "" : filePath, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_CHANNEL_CLOSED, errorNumber );        
        channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", filePath, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int iRes = 0;
    std::string fileName( filePath );

    // Endung gibt funktion vor.
    // Nur ".dfl"
    size_t pos = fileName.size() - 4;

    if ( fileName[pos] == '.' &&
        (((fileName[pos + 1] == 'd' || fileName[pos + 1] == 'D') &&
        (fileName[pos + 2] == 'f' || fileName[pos + 2] == 'F') &&
        (fileName[pos + 3] == 'l' || fileName[pos + 3] == 'L'))) )
    {
        // Textdaten uebertragen
        iRes = channelBridge->writeTextData( l_device_address, fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        { // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        { // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            iRes = 0;
        }
        else
        { // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }

        // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
        if ( iRes == 0 )
        {
            channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
        }
    } 
    else if ( fileName[pos] == '.' &&
        (((fileName[pos + 1] == 'd' || fileName[pos + 1] == 'D') &&
        (fileName[pos + 2] == 'f' || fileName[pos + 2] == 'F') &&
        (fileName[pos + 3] == 'k' || fileName[pos + 3] == 'K'))) )
    {
        // Touchdaten uebertragen
        iRes = channelBridge->writeTouchKeyData( l_device_address, fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        { // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        { // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF ) 
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                } 
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Modulfehler auswerten.
                channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            }
            iRes = 0;
        }
        else
        { // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }

        // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
        if ( iRes == 0 )
        {
            channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
        }
    } 
    else if ( fileName[pos] == '.' &&
        (((fileName[pos + 1] == 'd' || fileName[pos + 1] == 'D') &&
        (fileName[pos + 2] == 'f' || fileName[pos + 2] == 'F') &&
        (fileName[pos + 3] == 'c' || fileName[pos + 3] == 'C'))) )
    {
        // Farbdaten uebertragen
        iRes = channelBridge->writeDisplayColors( l_device_address, fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        { // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        { // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF ) 
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                } 
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Modulfehler auswerten.
                channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            }
            iRes = 0;
        }
        else
        { // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }

        // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
        if ( iRes == 0)
        {
            channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
        }
    } 
    else
    {
        int l_error_number = IV_ERROR_SUCCESS;
        bool selfExtracted = false;
        std::string localPath = fileName;

        // Für UNIX Systeme wird das DFZ-Archiv zunächst entpackt und als Ordner dem nachfolgenden Verarbeitungsschritt bereitgestellt.     
#if !defined(_MSC_VER) && !defined(__MINGW32__)
        localPath = Utils::uniqueTempDirectoryPath( "df_" );

        //std::string systemCall = "unzip -oqq " + fileName + " -d " + localPath;
        std::string systemCall = "unzip " + fileName + " -d " + localPath;
        int systemCallRes = system( systemCall.c_str() );
        channelBridge->logAddInfo( "cmd: '%s' - Result: %d", systemCall.c_str(), systemCallRes );

        if ( systemCallRes != 0 )
        {
            iRes = 0;
            l_error_number = IV_ERROR_PROCESSING;
        }
        else
        {
            selfExtracted = true;
        }
#endif

        // Daten uebertragen
        if ( l_error_number == IV_ERROR_SUCCESS )
        {
            std::string path = localPath;
            iRes = channelBridge->writeFileDevice( l_device_address, path, &l_error_number );
        }

        // Der temporär erstellte Ordner wird wieder gelöscht.
#if !defined(_MSC_VER) && !defined(__MINGW32__)
        if ( selfExtracted )
        {
            systemCall = "rm -rf " + localPath;
            systemCallRes = system( systemCall.c_str() );
            channelBridge->logAddInfo( "cmd: '%s' - result: %d", systemCall.c_str(), systemCallRes );
            
            if ( systemCallRes != 0 )
            {
                iRes = 0;
                l_error_number = IV_ERROR_PROCESSING;
            }
        }
#endif

        // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
        if ( iRes == 0 )
        {
            channelBridge->setLastErrorNumber( l_device_address, l_error_number, errorNumber );
            channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
        }
    }

    SoftwarebuildVersion &minVersion = channelBridge->minimalUpdateVersion();
    std::list<std::string> &blacklist = channelBridge->blacklistVersions();

    if ( !blacklist.empty() )
    {
        channelBridge->logAddInfo( "Blacklist: " );
        for ( const auto &item : blacklist )
        {
            channelBridge->logAddInfo( "%s,", item.c_str() );
        }
    }
    channelBridge->logAddInfo( "minimale Version: %s", (const char *)minVersion );


    if ( (iRes == 0) && (channelBridge->getLastErrorNumber( l_device_address ) == IV_ERROR_DEVICEFILE_MATCH) )
    {
        // Zusatzinfo in Logdatei schreiben
        int count = 0;
        std::string errorText( channelBridge->GetFileDeviceError() );
        std::vector<std::string> result;
        boost::split( result, errorText, [](char c) { return c == '\n'; } );
        for ( std::string &token : result )
        {
            channelBridge->logAddError( "%i\t%s", ++count, token.data() );
            channelBridge->logEnd( "" );
        }

        // Hardwareinfo als zusaetzliche Information rausgeben
        errorText = channelBridge->GetFileDeviceInfo();
        if ( errorText.size() )
        {
            count = 0;
            result.clear();
            boost::split( result, errorText, [](char c) { return c == '\n'; } );
            for ( std::string &token : result )
            {
                channelBridge->logAddInfo( "%i\t%s", ++count, token.data() );
                channelBridge->logEnd( "" );
            }
        }
    }

    channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", fileName.c_str(), (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( l_device_address ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCUploadModule( int connectionNumber, int deviceAddress, int fileType, const char *filePath, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "\t%i", fileType );

    // Geräteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 || fileType != 0 || filePath == nullptr )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", (filePath == nullptr) ? "" : filePath, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", filePath, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );
    int iRes = 0;

    // Dateityp ermitteln
    struct stat st;
    if ( stat( filePath, &st ) == -1 )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FILE_OPEN, errorNumber );
        return iRes;
    }
    // Nur ".up3" && ".up4" && ".up5" gueltig.
    size_t pos = strlen( filePath ) - 4;
    if (
#if defined(_MSC_VER) || defined(__MINGW32__)
        !((st.st_mode & _S_IFMT) == _S_IFREG) 
#else
        !((st.st_mode & S_IFMT) == S_IFREG) 
#endif
        || filePath[pos] != '.' || 
        !(filePath[pos + 1] == 'u' || filePath[pos + 1] == 'U') ||
        !(filePath[pos + 2] == 'p' || filePath[pos + 2] == 'P') ||
        !(filePath[pos + 3] == '3' || filePath[pos + 3] == '4' || filePath[pos + 3] == '5') )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FILE_EXTENSION, errorNumber );
        channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", filePath, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return iRes;
    }

    int l_error_number = IV_ERROR_SUCCESS;
    // Daten uebertragen
    iRes = channelBridge->writeModule( l_device_address, fileType, filePath, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( l_device_address, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
    }

    channelBridge->logEnd( "\t%s\t%i\t(%i, %i)", filePath, (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( l_device_address ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCModifyStudioFile( const char *sourceFilePath, const char *destFilePath, const char *xmlData, int flags, int *errorNumber )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *channelBridge = theApp.channelBridge( id );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBeginPure( __func__, "" );

    // zulaessige Bereiche pruefen
    if ( sourceFilePath == nullptr || destFilePath == nullptr || xmlData == nullptr )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%s\t%s\t%s\t%i\t(%i, %i)", (sourceFilePath != nullptr) ? sourceFilePath : "", (destFilePath != nullptr) ? destFilePath : "", (xmlData != nullptr) ? xmlData : "", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS, errorNumber );
    
    int l_error_number = IV_ERROR_SUCCESS;
    int iRes = channelBridge->modifyStudioFile( sourceFilePath, destFilePath, xmlData, flags, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( 0, l_error_number, errorNumber );
    }

    channelBridge->logEnd( "\t%s\t%s\t%s\t%i\t(%i, %i)", sourceFilePath, destFilePath, xmlData, (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCFileUpload( int connectionNumber, int deviceAddress, int fileType, const char *filePath, int flags, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "\t%i", fileType );

    // Geräteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0
        || filePath == nullptr  
        || fileType < -1 || fileType > 4 )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", (filePath == nullptr) ? "" : filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_CHANNEL_CLOSED, errorNumber );       
        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Blocktransfer besteht bereits? Nur bei Uebertragung von Dateidaten wird extra geprüft.
    if ( channelBridge->getBlockTransferState( l_device_address ) == ControllerSegment::BlockTransferState::active && fileType != 4 )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_BLOCK_TRANSFER_ACTIVE, errorNumber );
        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int iRes = 0;
    char _fileName[0x200];
    memset( _fileName, 0, sizeof( _fileName ) );
    strncpy( _fileName, filePath, sizeof( _fileName ) - 1 );

    // Endung gibt funktion vor.
    // Nur ".dfl"
    size_t pos = strlen( _fileName ) - 4;

    if ( fileType == 0 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'l' || _fileName[pos + 3] == 'L')) )
    {
        // Textdaten uebertragen
        iRes = channelBridge->writeTextData( l_device_address, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }
    }
    else if ( fileType == 1 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'k' || _fileName[pos + 3] == 'K')) )
    {
        // Touchdaten uebertragen
        iRes = channelBridge->writeTouchKeyData( l_device_address, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 ) 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                }
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Modulfehler auswerten.
                channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }
    }
    else if ( fileType == 2 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'c' || _fileName[pos + 3] == 'C')) )
    {
        // Farbdaten uebertragen
        iRes = channelBridge->writeDisplayColors( l_device_address, filePath );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 )
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                }
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Modulfehler auswerten.
                channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            }

            iRes = 0;

        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }
    }
    else if ( fileType == 3 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == '0' || _fileName[pos + 3] == '0')) )
    {
        // WLAN-Konfiguration uebertragen
        iRes = channelBridge->writeWlanConfigData( l_device_address, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 )
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                }
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Modulfehler auswerten.
                channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }
    }
    else if ( (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == '1' || _fileName[pos + 3] == '1')) )
    {
        // Hardwarekonfiguration uebertragen
        iRes = channelBridge->writeHardwareConfigData( l_device_address, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 )
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                }
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Modulfehler auswerten.
                channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }
    }
    else if ( fileType == 4 )
    {
        // Anderer Blocktransfer besteht bereits?
        if ( channelBridge->getBlockTransferState( l_device_address ) == ControllerSegment::BlockTransferState::active
            && channelBridge->getBlockTransferType( l_device_address ) != ControllerSegment::btt_write_file )
        {
            channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_BLOCK_TRANSFER_ACTIVE, errorNumber );
            channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
            return 0;
        }

        // Datei beliebiger Daten uebertragen
        iRes = channelBridge->writeFileData( l_device_address, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 )
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                }
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Modulfehler auswerten.
                channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }
    }
    else
    {
        // Angegebener Type oder Dateiendung ist unbekannt.
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FILE_EXTENSION, errorNumber );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
    }

    channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( l_device_address ) );

    return iRes;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCFileDownload( int connectionNumber, int deviceAddress, int fileType, const char *filePath, int flags, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "\t%i", fileType );

    // Geräteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0
        || filePath == nullptr 
        || fileType < -1  || fileType > 4 )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", (filePath == nullptr) ? "" : filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Blocktransfer besteht bereits?
    if ( channelBridge->getBlockTransferState( l_device_address ) == ControllerSegment::BlockTransferState::active && fileType != 4 )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_BLOCK_TRANSFER_ACTIVE, errorNumber );
        channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int iRes = 0;
    char _fileName[0x200];
    memset( _fileName, 0, sizeof( _fileName ) );
    strncpy( _fileName, filePath, sizeof( _fileName ) - 1 );

    // Endung kann funktion vorgeben.
    size_t pos = strlen( _fileName ) - 4;

    if ( fileType == 0 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'l' || _fileName[pos + 3] == 'L')) )
    {
        // Textdaten lesen
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
        iRes = 0;
    }
    else if ( fileType == 1 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'k' || _fileName[pos + 3] == 'K')) )
    {
        // Touchdaten uebertragen
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
        iRes = 0;
    }
    else if ( fileType == 2 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == 'c' || _fileName[pos + 3] == 'C')) )
    {
        // Farbdaten uebertragen
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
        iRes = 0;
    }
    else if ( fileType == 3 || (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == '0' || _fileName[pos + 3] == '0')) )
    {
        // WLAN-Konfiguration uebertragen
        iRes = channelBridge->readWlanConfigData( l_device_address, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 )
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                }
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Modulfehler auswerten.
                channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            }
            
            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }
    }
    else if ( (fileType == -1 && _fileName[pos] == '.' 
        && (_fileName[pos + 1] == 'd' || _fileName[pos + 1] == 'D') 
        && (_fileName[pos + 2] == 'f' || _fileName[pos + 2] == 'F') 
        && (_fileName[pos + 3] == '1' || _fileName[pos + 3] == '1')) )
    {
        // Hardware-Konfiguration Daten uebertragen
        iRes = channelBridge->readHardwareConfigData( l_device_address, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( (iRes >> 8) & 0xFF )
            {
                if ( ((iRes >> 8) & 0xFF) == 2 )
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                }
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Modulfehler auswerten.
                channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
            }
            
            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }
    }
    else if ( fileType == 4 )
    {
        // Anderer Blocktransfer besteht bereits?
        if ( channelBridge->getBlockTransferState( l_device_address ) == ControllerSegment::BlockTransferState::active
            && channelBridge->getBlockTransferType( l_device_address ) != ControllerSegment::btt_read_file )
        {
            channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_BLOCK_TRANSFER_ACTIVE, errorNumber );
            channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( l_device_address ) );
            return 0;
        }

        // Datei mit beliebigen Daten uebertragen
        iRes = channelBridge->readFileData( l_device_address, _fileName );
        if ( iRes == IV_ERROR_SUCCESS )
        {
            // Kein Fehler
            iRes = 1;
        }
        else if ( iRes > IV_ERROR_SUCCESS )
        {
            // Fehler bei Durchfuehrung des Befehls
            // Modulfehler auswerten.
            if ( IV_RESULT_MODULE( iRes ) )
            {
                if ( IV_RESULT_MODULE( iRes ) == 2 )
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_UNAVAILABLE, errorNumber );
                }
                else 
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }
            else
            {
                // Funktionsfehler auswerten.
                if ( IV_RESULT_FUNCTION( iRes ) == 12 )
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FILE_NOT_FOUND, errorNumber );
                }
                else if ( IV_RESULT_FUNCTION( iRes ) == 13 )
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FILE_WILL_BE_CREATED, errorNumber );
                }
                else
                {
                    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_PROCESSING, errorNumber );
                }
            }

            iRes = 0;
        }
        else
        {
            // Fehler waehrend Kommunikation oder Verarbeitung in DLL
            channelBridge->setLastErrorNumber( l_device_address, -iRes, errorNumber );
            iRes = 0;
        }
    }
    else
    {
        // Angegebener Type oder Dateiendung ist unbekannt.
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FILE_EXTENSION, errorNumber );
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( l_device_address ) );
    }

    channelBridge->logEnd( "\t%s\t%i\t%i\t(%i, %i)", filePath, flags, (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( l_device_address ) );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCConvertFile( const char *srcFilePath, int srcType, const char *dstFilePath, int dstType, int flags )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *channelBridge = theApp.channelBridge( id );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        return 0;
    }

    channelBridge->logBeginPure( __func__, "" );

    // zulaessige Bereiche pruefen
    if ( srcFilePath == nullptr || dstFilePath == nullptr || (srcType < 0 || srcType > 2) || (dstType < 0 || dstType > 2) || (flags < 0 || flags > 0) )
    {
        channelBridge->setLastErrorNumber( 0, IV_ERROR_FUNCTION_PARAMETER );
        channelBridge->logEnd( "\t%s\t%i\t%s\t%i\t%i\t(%i, %i)", (srcFilePath != nullptr) ? srcFilePath : "", srcType, (dstFilePath != nullptr) ? dstFilePath : "", dstType, flags, 0, channelBridge->getLastErrorNumber( 0 ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( 0, IV_ERROR_SUCCESS );

    int l_error_number = IV_ERROR_SUCCESS;
    int iRes = channelBridge->convertFile( srcFilePath, static_cast<FileType>(srcType), dstFilePath, static_cast<FileType>(dstType), flags, &l_error_number );

    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes != 1 )
    {
        channelBridge->setLastErrorNumber( 0, l_error_number );
    }

    channelBridge->logEnd( "\t%s\t%i\t%s\t%i\t%i\t(%i, %i)", srcFilePath, srcType, dstFilePath, dstType, flags, iRes, channelBridge->getLastErrorNumber( 0 ) );

    return iRes;
}
