#include "api/df_system_records.h"
#include "datafox_library_iv.h"

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSystemReadRecord( int connectionNumber, int deviceAddress, unsigned char *buffer, int *bufferSize, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return -1;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0
        || buffer == nullptr || bufferSize == nullptr || *bufferSize < IV_SYS_RECORDS_MAX )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t\t\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    uint16_t l_bufferSize = (*bufferSize > 0xFFFF) ? 0xFFFF : static_cast<uint16_t>(*bufferSize);
    int res = channelBridge->systemReadRecord( deviceAddress, buffer, l_bufferSize, l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( res == -1 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
        
        channelBridge->logContinue( "\t\t" );
    }
    else
    {
        *bufferSize = l_bufferSize;

        // Daten fuer Log in Puffer uebergeben
        unsigned char ucTmp[IV_SYS_RECORDS_MAX * 3 + 1];
        int i = 0;
        if ( res && channelBridge->logEnabled() )
        {
            for ( ; i < *bufferSize && i < static_cast<int>(sizeof( ucTmp ) - 3); ++i )
            {
                sprintf( (char *)&ucTmp[i * 3], "%02X ", (unsigned char)buffer[i] );
            }
        }
        ucTmp[i * 3] = 0;

        channelBridge->logContinue( "\t%s\t%i", ucTmp, *bufferSize );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, res, channelBridge->getLastErrorNumber( deviceAddress ) );

    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSystemQuitRecord( int connectionNumber, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return -1;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, -1, channelBridge->getLastErrorNumber( deviceAddress ) );
        return -1;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    int res = channelBridge->systemQuitRecord( deviceAddress, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( res == -1 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, res, channelBridge->getLastErrorNumber( deviceAddress ) );

    return res;
}

DF_API_EXTERN_DECORATE int DF_API_CALL DFCSystemRestoreRecords( int connectionNumber, int deviceAddress, int *errorNumber )
{
    ChannelBridge *channelBridge = theApp.channelBridge( connectionNumber );
    // Wurde Channel-Objekt ermittelt.
    if ( channelBridge == nullptr )
    {
        if ( errorNumber != nullptr )
        {
            *errorNumber = IV_ERROR_CHANNEL;
        }

        return 0;
    }

    channelBridge->logBegin( __func__, deviceAddress, "" );

    // Gerteadresse auf lokale mappen.
    uint8_t l_device_address = channelBridge->cast_device_address( deviceAddress );

    // zulaessige Bereiche pruefen
    if ( l_device_address == 0 )
    {
        channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_FUNCTION_PARAMETER, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Schnittstelle geoeffnet?
    if ( channelBridge->isOpen() == false )
    {
        channelBridge->setLastErrorNumber( deviceAddress, IV_ERROR_CHANNEL_CLOSED, errorNumber );
        channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, 0, channelBridge->getLastErrorNumber( deviceAddress ) );
        return 0;
    }

    // Fehlerwert zuruecksetzen.
    channelBridge->setLastErrorNumber( l_device_address, IV_ERROR_SUCCESS, errorNumber );

    int l_error_number = IV_ERROR_SUCCESS;
    int iRes = channelBridge->systemRestoreRecords( deviceAddress, &l_error_number );
    // Nach Kommunikationsfehler, Kanal bereinigen und ggf. Fehlernummer uebergeben.
    if ( iRes == 0 )
    {
        channelBridge->setLastErrorNumber( deviceAddress, l_error_number, errorNumber );
        channelBridge->resetWithError( channelBridge->getLastErrorNumber( deviceAddress ) );
    }

    channelBridge->logEnd( "\t%i\t(%i, %i)", (errorNumber == nullptr) ? 0 : *errorNumber, iRes, channelBridge->getLastErrorNumber( deviceAddress ) );

    return iRes;
}
