/*###########################################################*/
/** \file aes_crypt.h
Autor                 : Bernd Ottmann             01/2012
*/  
/*###########################################################*/
#pragma once
#ifndef DATAFOX_COMMUNICATION_AES_CRYPT_H
#define DATAFOX_COMMUNICATION_AES_CRYPT_H

#include <cstdint>

namespace Datafox
{
    namespace Communication
    {
        // Klasse zur Verschluesselung von Daten mittels AES.
        class AesCrypt
        {
        public:
            AesCrypt()
            {
                m_useInitVectorsAes = 1;
            }
        private:
            static const uint8_t m_sbox[256];
            static const uint8_t m_invSbox[256];
            static const uint8_t m_ffPow[256];
            static const uint8_t m_ffLog[256];
            static const uint8_t m_crcTab[256];
            static const uint8_t m_aesStartCrcValue = 0x53;
            uint8_t m_defaultKeyAes[16]; ///< Schluessel für Verschluesselung
            uint8_t m_initVectorEncryptAes[16]; ///< Speicher fuer Initialisierungs Vektor
            uint8_t m_initVectorDecryptAes[16]; ///< Speicher fuer Initialisierungs Vektor
            uint8_t m_useInitVectorsAes;  ///< benutze Initialisierungs Vektor beim Verschluesseln
            uint8_t m_roundKeysAes[176]; ///< Speicher fuer die einzelnen Round Keys

        public:
            // High Level Funktionen fuer die Verschluesselung
            void setKey( uint8_t *key_ptr, uint8_t size );
            uint16_t encrypt( uint8_t *data, uint16_t raw_size ); // Verschluesselung von Nutzdaten
            uint32_t encryptStream( uint8_t *data, uint32_t raw_size ); // Verschluesselung von Nutzdaten
            uint8_t decrypt( uint8_t *data, uint16_t *size );    // Entschluesselung von Nutzdaten
            uint8_t decryptStream( uint8_t *data, uint32_t *size );    // Entschluesselung von Nutzdaten

        private:
            // Checksumme fuer Verschluesselungsbereich berechnen
            uint8_t crcCheckData( uint8_t crc_previous, uint32_t size, uint8_t *data );
            // Blockweise
            void encryptBlock16( uint8_t *block ); // Verschluesselung von 16Byte Nutzdaten
            void decryptBlock16( uint8_t *block ); // Entschluesselung von 16Byte Nutzdaten
            // Helper, interne Berechnungsfunktionen
            void expandKey( uint8_t *input_key_ptr );
            void xOr( uint8_t *state, uint8_t *round_key, uint8_t size = 16 );
            void shiftRows( uint8_t *block );
            void invShiftRows( uint8_t *block );
            uint8_t ffMul( uint8_t a, uint8_t b );
            void mixColumn( uint8_t *block );
            void invMixColumn( uint8_t *block );
        };
    } // namespace Communication
} // namespace Datafox

#endif
